@@ -4,9 +4,9 @@
  *
  * Author :     Paul Obermeier (paul@poSoft.de)
  *
- * Date :       Die Feb 20 14:27:18 CET 2001
+ * Date :       2001 / 02 / 20
  *
- * Copyright :  (C) 2001-2002 Paul Obermeier
+ * Copyright :  (C) 2001-2019 Paul Obermeier
  *
  * Description :
  *
@@ -38,7 +38,7 @@
  * -verbose <bool>:     If set to true, additional information about the file
  *                      format is printed to stdout. Default is "false".
  * -compression <type>: Set the compression mode to either "none" or "rle".
- *			Default is "rle".
+ *                      Default is "rle".
  *
  * Notes:
  *
@@ -66,51 +66,47 @@
 /* Some defines and typedefs. */
 #define TRUE  1
 #define FALSE 0
-typedef unsigned char Boln;	/* Boolean value: TRUE or FALSE */
-typedef unsigned char UByte;	/* Unsigned  8 bit integer */
-typedef char  Byte;		/* Signed    8 bit integer */
-typedef unsigned short UShort;	/* Unsigned 16 bit integer */
-typedef short Short;		/* Signed   16 bit integer */
-typedef int Int;		/* Signed   32 bit integer */
+typedef unsigned char Boln;     /* Boolean value: TRUE or FALSE */
+typedef unsigned char UByte;    /* Unsigned  8 bit integer */
+typedef char  Byte;             /* Signed    8 bit integer */
+typedef unsigned short UShort;  /* Unsigned 16 bit integer */
+typedef short Short;            /* Signed   16 bit integer */
+typedef int Int;                /* Signed   32 bit integer */
 
 typedef struct {
-  UByte manufacturer;
-  UByte version;
-  UByte compression;
-  UByte bpp;
-  Short x1;
-  Short y1;
-  Short x2;
-  Short y2;
-  Short hdpi;
-  Short vdpi;
-  UByte colormap[48];
-  UByte reserved;
-  UByte planes;
-  Short bytesperline;
-  Short color;
-  UByte filler[58];
+    UByte manufacturer;
+    UByte version;
+    UByte compression;
+    UByte bpp;
+    Short x1;
+    Short y1;
+    Short x2;
+    Short y2;
+    Short hdpi;
+    Short vdpi;
+    UByte colormap[48];
+    UByte reserved;
+    UByte planes;
+    Short bytesperline;
+    Short color;
+    UByte filler[58];
 } PCXHEADER;
 
-/* This function determines at runtime, whether we have to swap bytes.
-   The PCX image format expects data to be in Intel (Little-endian) format. */
-
-static int isIntel (void)
-{
-    unsigned long val = 513;
-    /* On Intel (little-endian) systems this value is equal to "\01\02\00\00".
-       On big-endian systems this value equals "\00\00\02\01" */
-    return memcmp(&val, "\01\02", 2) == 0;
-}
+/* Format options structure for use with ParseFormatOpts */
+typedef struct {
+    Int   compression;
+    Boln  verbose;
+} FMTOPT;
 
 #define htoqs(x) qtohs(x)
 static UShort qtohs (UShort x)
 {
-    if (!isIntel ()) {
-	return ((UShort)((((UShort)(x) & 0x00ff) << 8) | \
-			 (((UShort)(x) & 0xff00) >> 8)));
+    /* The PCX image format expects data to be in Intel (Little-endian) format. */
+    if (!tkimg_IsIntel ()) {
+        return ((UShort)((((UShort)(x) & 0x00ff) << 8) | \
+                         (((UShort)(x) & 0xff00) >> 8)));
     } else {
-	return x;
+        return x;
     }
 }
 
@@ -120,8 +116,9 @@ static UShort qtohs (UShort x)
 static Boln readUByte (tkimg_MFile *handle, UByte *b)
 {
     char buf[1];
-    if (1 != tkimg_Read2(handle, (char *) buf, 1))
+    if (1 != tkimg_Read2(handle, (char *) buf, 1)) {
         return FALSE;
+    }
     *b = buf[0];
     return TRUE;
 }
@@ -136,19 +133,20 @@ static Boln writeUByte (tkimg_MFile *handle, UByte b)
 {
     UByte buf[1];
     buf[0] = b;
-    if (1 != tkimg_Write2(handle, (const char *)buf, 1))
+    if (1 != tkimg_Write2(handle, (const char *)buf, 1)) {
         return FALSE;
+    }
     return TRUE;
 }
 
 static Boln read_pcx_header (tkimg_MFile *ifp, PCXHEADER *pcxhdr)
 {
     if (tkimg_Read2(ifp, (char *)pcxhdr, 128) != 128) {
-	return FALSE;
+        return FALSE;
     }
 
     if (pcxhdr->manufacturer != 10) {
-	return FALSE;
+        return FALSE;
     }
     if (pcxhdr->bpp != 1 && pcxhdr->bpp != 8) {
         return FALSE;
@@ -188,27 +186,27 @@ static Boln readline (tkimg_MFile *handle, UByte *buffer, Int bytes, Int compr)
     static UByte count = 0, value = 0;
 
     if (compr) {
-	while (bytes--) {
-	    if (count == 0) {
-	        if (!readUByte (handle, &value)) {
-		    return TRUE;
-		}
-	        if (value < 0xc0) {
-		    count = 1;
-		} else {
-		    count = value - 0xc0;
-		    if (!readUByte (handle, &value)) {
-			return TRUE;
-		    }
-		}
-	    }
-	    count--;
-	    *(buffer++) = value;
-	}
+        while (bytes--) {
+            if (count == 0) {
+                if (!readUByte (handle, &value)) {
+                    return TRUE;
+                }
+                if (value < 0xc0) {
+                    count = 1;
+                } else {
+                    count = value - 0xc0;
+                    if (!readUByte (handle, &value)) {
+                        return TRUE;
+                    }
+                }
+            }
+            count--;
+            *(buffer++) = value;
+        }
     } else {
-	if (bytes != tkimg_Read2(handle, (char *)buffer, bytes)) {
-	    return FALSE;
-	}
+        if (bytes != tkimg_Read2(handle, (char *)buffer, bytes)) {
+            return FALSE;
+        }
     }
     return TRUE;
 }
@@ -223,22 +221,22 @@ static Boln writeline (tkimg_MFile *handle, UByte *buffer, Int bytes)
         count = 1;
 
         while (buffer < finish && count < 63 && *buffer == value) {
-	    count++;
+            count++;
             buffer++;
-	}
+        }
 
         if (value < 0xc0 && count == 1) {
-	    if (!writeUByte (handle, value)) {
-		return FALSE;
-	    }
-	} else {
-	    if (!writeUByte (handle, 0xc0 + count)) {
-		return FALSE;
-	    }
-	    if (!writeUByte (handle, value)) {
-		return FALSE;
-	    }
-	}
+            if (!writeUByte (handle, value)) {
+                return FALSE;
+            }
+        } else {
+            if (!writeUByte (handle, 0xc0 + count)) {
+                return FALSE;
+            }
+            if (!writeUByte (handle, value)) {
+                return FALSE;
+            }
+        }
     }
     return TRUE;
 }
@@ -279,15 +277,15 @@ static Boln load_8 (Tcl_Interp *interp, tkimg_MFile *ifp,
     /* Read in the whole image data as indices. */
     for (y=0; y<stopY; y++) {
         if (!readline (ifp, line, bytesPerLine, compr)) {
-	    ckfree ((char *) line);
-	    ckfree ((char *) buffer);
-	    ckfree ((char *) indBuf);
+            ckfree ((char *) line);
+            ckfree ((char *) buffer);
+            ckfree ((char *) indBuf);
             sprintf(errMsg, "Unexpected end-of-file while scanline %d", y);
             Tcl_AppendResult(interp, errMsg, (char *)NULL);
-	    return FALSE;
-	}
+            return FALSE;
+        }
         memcpy (indBufPtr, line, fileWidth);
-	indBufPtr += fileWidth;
+        indBufPtr += fileWidth;
     }
     /* Read the colormap: 256 entries a 3 values for RGB */
     if (tkimg_Read2(ifp, (char *)&sepChar, 1) == 1) {
@@ -298,7 +296,7 @@ static Boln load_8 (Tcl_Interp *interp, tkimg_MFile *ifp,
                 ckfree ((char *) buffer);
                 ckfree ((char *) indBuf);
                 Tcl_AppendResult (interp, "Unexpected end-of-file while reading colormap",
-		                 (char *) NULL);
+                                 (char *) NULL);
                 return FALSE;
             }
             haveColormap = TRUE;
@@ -321,7 +319,8 @@ static Boln load_8 (Tcl_Interp *interp, tkimg_MFile *ifp,
                 buffer[x * 3 + 2] = indBuf[y*fileWidth + x];
             }
         }
-        if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY, width, 1, TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
+        if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY, width, 1,
+            TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
             result = FALSE;
             break;
         }
@@ -362,23 +361,24 @@ static Boln load_24 (Tcl_Interp *interp, tkimg_MFile *ifp,
     outY  = destY;
 
     for (y=0; y<stopY; y++) {
-	for (c=0; c<3; c++) {
-	    if (!readline (ifp, line, bytesPerLine, compr)) {
-		ckfree ((char *) line);
-		ckfree ((char *) buffer);
-		return FALSE;
-	    }
-	    for (x=0; x<fileWidth; x++) {
-	        buffer[x * 3 + c] = line[x];
-	    }
-	}
-	if (y >= srcY) {
-	    if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY, width, 1, TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
-	    	result = FALSE;
-	    	break;
-	    }
-	    outY++;
-	}
+        for (c=0; c<3; c++) {
+            if (!readline (ifp, line, bytesPerLine, compr)) {
+                ckfree ((char *) line);
+                ckfree ((char *) buffer);
+                return FALSE;
+            }
+            for (x=0; x<fileWidth; x++) {
+                buffer[x * 3 + c] = line[x];
+            }
+        }
+        if (y >= srcY) {
+            if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY, width, 1,
+                TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
+                result = FALSE;
+                break;
+            }
+            outY++;
+        }
     }
     ckfree ((char *) line);
     ckfree ((char *) buffer);
@@ -415,21 +415,22 @@ static Boln load_1 (Tcl_Interp *interp, tkimg_MFile *ifp,
 
     for (y=0; y<stopY; y++) {
         if (!readline (ifp, line, bytesPerLine, compr)) {
-	    ckfree ((char *) line);
-	    ckfree ((char *) buffer);
+            ckfree ((char *) line);
+            ckfree ((char *) buffer);
             return FALSE;
         }
         for (x=0; x<fileWidth; x++) {
-	    if (line[x/8] & (128 >> (x%8))) {
-	        buffer[x] = 255;
-	    } else {
-	        buffer[x] = 0;
-	    }
-	}
+            if (line[x/8] & (128 >> (x%8))) {
+                buffer[x] = 255;
+            } else {
+                buffer[x] = 0;
+            }
+        }
         if (y >= srcY) {
-            if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY, width, 1, TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
-            	result = FALSE;
-            	break;
+            if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY, width, 1,
+                TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
+                result = FALSE;
+                break;
             }
             outY++;
         }
@@ -443,104 +444,72 @@ static Boln load_1 (Tcl_Interp *interp, tkimg_MFile *ifp,
  * Prototypes for local procedures defined in this file:
  */
 
-static int ParseFormatOpts(Tcl_Interp *interp, Tcl_Obj *format,
-                 int *comp, int *verb, int *matte);
+static int ParseFormatOpts(Tcl_Interp *interp, Tcl_Obj *format, FMTOPT *opts);
 static int CommonMatch(tkimg_MFile *handle, int *widthPtr,
-	         int *heightPtr, PCXHEADER *pcxHeaderPtr);
+                 int *heightPtr, PCXHEADER *pcxHeaderPtr);
 static int CommonRead(Tcl_Interp *interp, tkimg_MFile *handle,
-	         const char *filename, Tcl_Obj *format,
-	         Tk_PhotoHandle imageHandle, int destX, int destY,
-		 int width, int height, int srcX, int srcY);
+                 const char *filename, Tcl_Obj *format,
+                 Tk_PhotoHandle imageHandle, int destX, int destY,
+                 int width, int height, int srcX, int srcY);
 static int CommonWrite(Tcl_Interp *interp,
                  const char *filename, Tcl_Obj *format,
                  tkimg_MFile *handle, Tk_PhotoImageBlock *blockPtr);
 
-static int ParseFormatOpts (interp, format, comp, verb, matte)
+static int ParseFormatOpts (interp, format, opts)
     Tcl_Interp *interp;
     Tcl_Obj *format;
-    int *comp;
-    int *verb;
-    int *matte;
+    FMTOPT *opts;
 {
-    static const char *const pcxOptions[] = {"-compression", "-verbose", "-matte", NULL};
-    int objc, length, c, i, index;
+    static const char *const pcxOptions[] = {
+        "-compression", "-verbose", NULL 
+    };
+    int objc, i, index;
+    char *optionStr;
     Tcl_Obj **objv;
-    const char *compression, *verbose, *transp;
+    int boolVal;
 
-    *comp = 1;
-    *verb = 0;
-    *matte = 1;
-    if (tkimg_ListObjGetElements(interp, format, &objc, &objv) != TCL_OK)
-	return TCL_ERROR;
+    /* Initialize options with default values. */
+    opts->compression = 1;
+    opts->verbose     = 0;
+
+    if (tkimg_ListObjGetElements(interp, format, &objc, &objv) != TCL_OK) {
+        return TCL_ERROR;
+    }
     if (objc) {
-	compression = "rle";
-	verbose     = "0";
-	transp      = "1";
-	for (i=1; i<objc; i++) {
-	    if (Tcl_GetIndexFromObj(interp, objv[i], (const char *CONST86 *)pcxOptions,
-		    "format option", 0, &index) != TCL_OK) {
-		return TCL_ERROR;
-	    }
-	    if (++i >= objc) {
-		Tcl_AppendResult(interp, "No value for option \"",
-			Tcl_GetStringFromObj (objv[--i], (int *) NULL),
-			"\"", (char *) NULL);
-		return TCL_ERROR;
-	    }
-	    switch(index) {
-		case 0:
-		    compression = Tcl_GetStringFromObj(objv[i], (int *) NULL);
-		    break;
-		case 1:
-		    verbose = Tcl_GetStringFromObj(objv[i], (int *) NULL);
-		    break;
-		case 2:
-		    transp = Tcl_GetStringFromObj(objv[i], (int *) NULL);
-		    break;
-	    }
-	}
-
-	c = compression[0]; length = strlen (compression);
-	if ((c == 'n') && (!strncmp (compression, "none", length))) {
-	    *comp = 0;
-	} else if ((c == 'r') && (!strncmp (compression, "rle",length))) {
-	    *comp = 1;
-	} else {
-	    Tcl_AppendResult(interp, "invalid compression mode \"",
-		    compression, "\": should be rle or none", (char *) NULL);
-	    return TCL_ERROR;
-	}
-
-	c = verbose[0]; length = strlen (verbose);
-	if (!strncmp (verbose, "1", length) || \
-	    !strncmp (verbose, "true", length) || \
-	    !strncmp (verbose, "on", length)) {
-	    *verb = 1;
-	} else if (!strncmp (verbose, "0", length) || \
-	    !strncmp (verbose, "false", length) || \
-	    !strncmp (verbose, "off", length)) {
-	    *verb = 0;
-	} else {
-	    Tcl_AppendResult(interp, "invalid verbose mode \"", verbose,
-                              "\": should be 1 or 0, on or off, true or false",
-			      (char *) NULL);
-	    return TCL_ERROR;
-	}
-
-        c = transp[0]; length = strlen (transp);
-        if (!strncmp (transp, "1", length) || \
-            !strncmp (transp, "true", length) || \
-            !strncmp (transp, "on", length)) {
-            *matte = 1;
-        } else if (!strncmp (transp, "0", length) || \
-            !strncmp (transp, "false", length) || \
-            !strncmp (transp, "off", length)) {
-            *matte = 0;
-        } else {
-            Tcl_AppendResult(interp, "invalid alpha (matte) mode \"", verbose,
-                              "\": should be 1 or 0, on or off, true or false",
-                              (char *) NULL);
-            return TCL_ERROR;
+        for (i=1; i<objc; i++) {
+            if (Tcl_GetIndexFromObj(interp, objv[i], (const char *CONST86 *)pcxOptions,
+                    "format option", 0, &index) != TCL_OK) {
+                return TCL_ERROR;
+            }
+            if (++i >= objc) {
+                Tcl_AppendResult(interp, "No value for option \"",
+                        Tcl_GetStringFromObj (objv[--i], (int *) NULL),
+                        "\"", (char *) NULL);
+                return TCL_ERROR;
+            }
+            optionStr = Tcl_GetStringFromObj(objv[i], (int *) NULL);
+            switch(index) {
+                case 0:
+                    if (!strncmp (optionStr, "none", strlen ("none"))) {
+                        opts->compression = 0;
+                    } else if (!strncmp (optionStr, "rle", strlen ("rle"))) {
+                        opts->compression = 1;
+                    } else {
+                        Tcl_AppendResult (interp, "Invalid compression mode \"", optionStr,
+                                          "\": Must be \"none\" or \"rle\"", (char *) NULL);
+                        return TCL_ERROR;
+                    }
+                    break;
+                case 1:
+                    if (Tcl_GetBoolean(interp, optionStr, &boolVal) == TCL_ERROR) {
+                        Tcl_AppendResult (interp, "Invalid verbose mode \"", optionStr,
+                                          "\": should be 1 or 0, on or off, true or false",
+                                          (char *) NULL);
+                        return TCL_ERROR;
+                    }
+                    opts->verbose = boolVal;
+                    break;
+            }
         }
     }
     return TCL_OK;
@@ -572,7 +541,7 @@ static int ObjMatch(
     tkimg_MFile handle;
 
     if (!tkimg_ReadInit(data, 10, &handle)) {
-	return 0;
+        return 0;
     }
     return CommonMatch(&handle, widthPtr, heightPtr, NULL);
 }
@@ -586,39 +555,43 @@ static int CommonMatch(handle, widthPtr, heightPtr, pcxHeaderPtr)
     PCXHEADER ph;
     Int offset_x, offset_y;
 
-    if (!read_pcx_header (handle, &ph))
-	return 0;
+    if (!read_pcx_header (handle, &ph)) {
+        return 0;
+    }
 
     offset_x = qtohs (ph.x1);
     offset_y = qtohs (ph.y1);
 
-    if (offset_x < 0 || offset_y < 0)
-	return 0;
+    if (offset_x < 0 || offset_y < 0) {
+        return 0;
+    }
 
     *widthPtr  = qtohs (ph.x2) - offset_x + 1;
     *heightPtr = qtohs (ph.y2) - offset_y + 1;
 
-    if (*widthPtr < 1 || *heightPtr < 1)
-	return 0;
+    if (*widthPtr < 1 || *heightPtr < 1) {
+        return 0;
+    }
 
-    if (pcxHeaderPtr)
-	*pcxHeaderPtr = ph;
+    if (pcxHeaderPtr) {
+        *pcxHeaderPtr = ph;
+    }
     return 1;
 }
 
 static int ChnRead(interp, chan, filename, format, imageHandle,
-	            destX, destY, width, height, srcX, srcY)
-    Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
-    Tcl_Channel chan;		/* The image channel, open for reading. */
-    const char *filename;	/* The name of the image file. */
-    Tcl_Obj *format;		/* User-specified format object, or NULL. */
-    Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
-    int destX, destY;		/* Coordinates of top-left pixel in
-			         * photo image to be written to. */
-    int width, height;		/* Dimensions of block of photo image to
-			         * be written to. */
-    int srcX, srcY;		/* Coordinates of top-left pixel to be used
-				 * in image being read. */
+                   destX, destY, width, height, srcX, srcY)
+    Tcl_Interp *interp;         /* Interpreter to use for reporting errors. */
+    Tcl_Channel chan;           /* The image channel, open for reading. */
+    const char *filename;       /* The name of the image file. */
+    Tcl_Obj *format;            /* User-specified format object, or NULL. */
+    Tk_PhotoHandle imageHandle; /* The photo image to write into. */
+    int destX, destY;           /* Coordinates of top-left pixel in
+                                 * photo image to be written to. */
+    int width, height;          /* Dimensions of block of photo image to
+                                 * be written to. */
+    int srcX, srcY;             /* Coordinates of top-left pixel to be used
+                                 * in image being read. */
 {
     tkimg_MFile handle;
 
@@ -626,12 +599,12 @@ static int ChnRead(interp, chan, filename, format, imageHandle,
     handle.state = IMG_CHAN;
 
     return CommonRead (interp, &handle, filename, format,
-		       imageHandle, destX, destY,
-		       width, height, srcX, srcY);
+                       imageHandle, destX, destY,
+                       width, height, srcX, srcY);
 }
 
 static int ObjRead (interp, data, format, imageHandle,
-	            destX, destY, width, height, srcX, srcY)
+                    destX, destY, width, height, srcX, srcY)
     Tcl_Interp *interp;
     Tcl_Obj *data;
     Tcl_Obj *format;
@@ -644,84 +617,89 @@ static int ObjRead (interp, data, format, imageHandle,
 
     tkimg_ReadInit (data, 10, &handle);
     return CommonRead (interp, &handle, "InlineData", format, imageHandle,
-		       destX, destY, width, height, srcX, srcY);
+                       destX, destY, width, height, srcX, srcY);
 }
 
 static int CommonRead (interp, handle, filename, format, imageHandle,
-		       destX, destY, width, height, srcX, srcY)
+                       destX, destY, width, height, srcX, srcY)
     Tcl_Interp *interp;         /* Interpreter to use for reporting errors. */
     tkimg_MFile *handle;        /* The image file, open for reading. */
     const char *filename;       /* The name of the image file. */
     Tcl_Obj *format;            /* User-specified format object, or NULL. */
     Tk_PhotoHandle imageHandle; /* The photo image to write into. */
     int destX, destY;           /* Coordinates of top-left pixel in
-				 * photo image to be written to. */
+                                 * photo image to be written to. */
     int width, height;          /* Dimensions of block of photo image to
-			         * be written to. */
+                                 * be written to. */
     int srcX, srcY;             /* Coordinates of top-left pixel to be used
-			         * in image being read. */
+                                 * in image being read. */
 {
     int fileWidth, fileHeight;
     int outWidth, outHeight;
     int retCode = TCL_OK;
     PCXHEADER ph;
-    int compr, verbose, matte;
+    FMTOPT opts;
     char errMsg[200];
 
-    if (ParseFormatOpts(interp, format, &compr, &verbose, &matte) != TCL_OK) {
+    if (ParseFormatOpts(interp, format, &opts) != TCL_OK) {
         return TCL_ERROR;
     }
 
     CommonMatch(handle, &fileWidth, &fileHeight, &ph);
-    if (verbose)
+    if (opts.verbose) {
         printImgInfo (&ph, filename, "Reading image:");
+    }
 
     if ((srcX + width) > fileWidth) {
-	outWidth = fileWidth - srcX;
+        outWidth = fileWidth - srcX;
     } else {
-	outWidth = width;
+        outWidth = width;
     }
     if ((srcY + height) > fileHeight) {
-	outHeight = fileHeight - srcY;
+        outHeight = fileHeight - srcY;
     } else {
-	outHeight = height;
+        outHeight = height;
     }
     if ((outWidth <= 0) || (outHeight <= 0)
-	|| (srcX >= fileWidth) || (srcY >= fileHeight)) {
-	return TCL_OK;
+        || (srcX >= fileWidth) || (srcY >= fileHeight)) {
+        return TCL_OK;
     }
 
     if (tkimg_PhotoExpand(interp, imageHandle, destX + outWidth, destY + outHeight) == TCL_ERROR) {
-	return TCL_ERROR;
+        return TCL_ERROR;
     }
 
-    if (ph.compression)
-	tkimg_ReadBuffer (1);
+    if (ph.compression) {
+        tkimg_ReadBuffer (1);
+    }
 
     if (ph.planes == 1 && ph.bpp == 1) {
         if (!load_1 (interp, handle, imageHandle, destX, destY,
                      outWidth, outHeight, srcX, srcY, fileWidth, fileHeight,
-                     qtohs (ph.bytesperline), ph.compression))
-	    retCode = TCL_ERROR;
+                     qtohs (ph.bytesperline), ph.compression)) {
+            retCode = TCL_ERROR;
+        }
     } else if (ph.planes == 4 && ph.bpp == 1) {
-	Tcl_AppendResult(interp, "Format (4 channels, 1 bit per channel) ",
+        Tcl_AppendResult(interp, "Format (4 channels, 1 bit per channel) ",
                           "is not supported yet.", (char *)NULL);
-	retCode = TCL_ERROR;
+        retCode = TCL_ERROR;
     } else if (ph.planes == 1 && ph.bpp == 8) {
         if (!load_8 (interp, handle, imageHandle, destX, destY,
                      outWidth, outHeight, srcX, srcY, fileWidth, fileHeight,
-                     qtohs (ph.bytesperline), ph.compression))
-	    retCode = TCL_ERROR;
+                     qtohs (ph.bytesperline), ph.compression)) {
+            retCode = TCL_ERROR;
+        }
     } else if (ph.planes == 3 && ph.bpp == 8) {
         if (!load_24 (interp, handle, imageHandle, destX, destY,
                       outWidth, outHeight, srcX, srcY, fileWidth, fileHeight,
-                      qtohs (ph.bytesperline), ph.compression))
-	    retCode = TCL_ERROR;
+                      qtohs (ph.bytesperline), ph.compression)) {
+            retCode = TCL_ERROR;
+        }
     } else {
-	sprintf(errMsg, "Image has invalid channel/bpp combination: (%d, %d)",
-			  ph.planes, ph.bpp);
-	Tcl_AppendResult(interp, errMsg, (char *)NULL);
-	retCode = TCL_ERROR;
+        sprintf(errMsg, "Image has invalid channel/bpp combination: (%d, %d)",
+                          ph.planes, ph.bpp);
+        Tcl_AppendResult(interp, errMsg, (char *)NULL);
+        retCode = TCL_ERROR;
     }
     tkimg_ReadBuffer (0);
     return retCode;
@@ -739,7 +717,7 @@ static int ChnWrite (interp, filename, format, blockPtr)
 
     chan = tkimg_OpenFileChannel (interp, filename, 0644);
     if (!chan) {
-	return TCL_ERROR;
+        return TCL_ERROR;
     }
 
     handle.data = (char *) chan;
@@ -747,7 +725,7 @@ static int ChnWrite (interp, filename, format, blockPtr)
 
     result = CommonWrite (interp, filename, format, &handle, blockPtr);
     if (Tcl_Close(interp, chan) == TCL_ERROR) {
-	return TCL_ERROR;
+        return TCL_ERROR;
     }
     return result;
 }
@@ -767,9 +745,9 @@ static int StringWrite(
     tkimg_Putc(IMG_DONE, &handle);
 
     if (result == TCL_OK) {
-	Tcl_DStringResult(interp, &data);
+        Tcl_DStringResult(interp, &data);
     } else {
-	Tcl_DStringFree(&data);
+        Tcl_DStringFree(&data);
     }
     return result;
 }
@@ -786,10 +764,10 @@ static int CommonWrite (interp, filename, format, handle, blockPtr)
     UByte   *pixelPtr, *pixRowPtr;
     PCXHEADER ph;
     UByte *row;
-    int compr, verbose, matte; /* Format options */
+    FMTOPT opts;
     char errMsg[200];
 
-    if (ParseFormatOpts(interp, format, &compr, &verbose, &matte) != TCL_OK) {
+    if (ParseFormatOpts(interp, format, &opts) != TCL_OK) {
         return TCL_ERROR;
     }
 
@@ -807,14 +785,14 @@ static int CommonWrite (interp, filename, format, handle, blockPtr)
         alphaOffset = 0;
     }
 
-    nchan   = 3;
-    nBytes  = blockPtr->width * nchan;
+    nchan  = 3;
+    nBytes = blockPtr->width * nchan;
 
     /* Fill the PCX header struct and write the header to the channel. */
     memset (&ph, 0, sizeof (PCXHEADER));
     ph.manufacturer = 0x0a;
     ph.version = 5;
-    ph.compression = compr;
+    ph.compression = opts.compression;
     ph.bpp = 8;
     ph.planes = 3;
     ph.color = htoqs (1);
@@ -829,50 +807,51 @@ static int CommonWrite (interp, filename, format, handle, blockPtr)
     ph.reserved = 0;
 
     if (tkimg_Write2(handle, (const char *)&ph, 128) != 128) {
-	Tcl_AppendResult(interp, "Can't write PCX header.", (char *)NULL);
-	return TCL_ERROR;
+        Tcl_AppendResult(interp, "Can't write PCX header.", (char *)NULL);
+        return TCL_ERROR;
     }
 
     row = (UByte *) ckalloc (nBytes);
     /* Now write out the image data. */
     pixRowPtr = blockPtr->pixelPtr + blockPtr->offset[0];
-    if (!compr) {
-	for (y=0; y<blockPtr->height; y++) {
-	    pixelPtr = pixRowPtr;
-	    for (x=0; x<blockPtr->width; x++) {
-		row[x + 0*blockPtr->width] = pixelPtr[redOffset];
-		row[x + 1*blockPtr->width] = pixelPtr[greenOffset];
-		row[x + 2*blockPtr->width] = pixelPtr[blueOffset];
-		pixelPtr += blockPtr->pixelSize;
-	    }
-	    if (nBytes != tkimg_Write2(handle, (const char *)row, nBytes)) {
-		sprintf(errMsg, "Can't write %d bytes to image file.", nBytes);
-		Tcl_AppendResult(interp, errMsg, (char *)NULL);
-		ckfree ((char *)row);
-		return TCL_ERROR;
-	    }
-	    pixRowPtr += blockPtr->pitch;
-	}
-    } else { 			/* RLE compression */
-	for (y = 0; y < blockPtr->height; y++) {
-	    pixelPtr = pixRowPtr;
-	    for (x = 0; x < blockPtr->width; x++) {
-		row[x + 0*blockPtr->width] = pixelPtr[redOffset];
-		row[x + 1*blockPtr->width] = pixelPtr[greenOffset];
-		row[x + 2*blockPtr->width] = pixelPtr[blueOffset];
-		pixelPtr += blockPtr->pixelSize;
-	    }
-	    if (!writeline (handle, row, nBytes)) {
-		sprintf(errMsg, "Can't write %d bytes to image file.", nBytes);
-		Tcl_AppendResult(interp, errMsg, (char *)NULL);
-		ckfree ((char *)row);
-		return TCL_ERROR;
-	    }
-	    pixRowPtr += blockPtr->pitch;
-	}
+    if (!opts.compression) {
+        for (y=0; y<blockPtr->height; y++) {
+            pixelPtr = pixRowPtr;
+            for (x=0; x<blockPtr->width; x++) {
+                row[x + 0*blockPtr->width] = pixelPtr[redOffset];
+                row[x + 1*blockPtr->width] = pixelPtr[greenOffset];
+                row[x + 2*blockPtr->width] = pixelPtr[blueOffset];
+                pixelPtr += blockPtr->pixelSize;
+            }
+            if (nBytes != tkimg_Write2(handle, (const char *)row, nBytes)) {
+                sprintf(errMsg, "Can't write %d bytes to image file.", nBytes);
+                Tcl_AppendResult(interp, errMsg, (char *)NULL);
+                ckfree ((char *)row);
+                return TCL_ERROR;
+            }
+            pixRowPtr += blockPtr->pitch;
+        }
+    } else {                    /* RLE compression */
+        for (y = 0; y < blockPtr->height; y++) {
+            pixelPtr = pixRowPtr;
+            for (x = 0; x < blockPtr->width; x++) {
+                row[x + 0*blockPtr->width] = pixelPtr[redOffset];
+                row[x + 1*blockPtr->width] = pixelPtr[greenOffset];
+                row[x + 2*blockPtr->width] = pixelPtr[blueOffset];
+                pixelPtr += blockPtr->pixelSize;
+            }
+            if (!writeline (handle, row, nBytes)) {
+                sprintf(errMsg, "Can't write %d bytes to image file.", nBytes);
+                Tcl_AppendResult(interp, errMsg, (char *)NULL);
+                ckfree ((char *)row);
+                return TCL_ERROR;
+            }
+            pixRowPtr += blockPtr->pitch;
+        }
     }
-    if (verbose)
+    if (opts.verbose) {
         printImgInfo (&ph, filename, "Saving image:");
+    }
     ckfree ((char *)row);
     return TCL_OK;
 }
